@@ -17,7 +17,7 @@ from warnings import warn
 #site
 from pkg_resources import resource_string
 #libs
-from passlib.registry import get_crypt_handler
+from passlib.registry import get_crypt_handler, _unload_handler_name
 from passlib.utils import Undef, is_crypt_handler, splitcomma, rng
 #pkg
 #local
@@ -45,7 +45,7 @@ def _parse_policy_key(key):
     elif len(parts) == 3:
         cat, name, opt = parts
     else:
-        raise KeyError, "keys must have 0..2 separators: %r" % (orig,)
+        raise KeyError("keys must have 0..2 separators: %r" % (orig,))
     if cat == "default":
         cat = None
     assert name
@@ -77,13 +77,13 @@ def parse_policy_items(source):
         cat, name, opt = _parse_policy_key(key)
         if name == "context":
             if cat and opt == "schemes":
-                raise KeyError, "current code does not support per-category schemes"
+                raise KeyError("current code does not support per-category schemes")
                 #NOTE: forbidding this because it would really complicate the behavior
                 # of CryptContext.identify & CryptContext.lookup.
                 # most useful behaviors here can be had by overridding deprecated and default, anyways.
         else:
             if opt == "salt":
-                raise KeyError, "'salt' option is not allowed to be set via a policy object"
+                raise KeyError("'salt' option is not allowed to be set via a policy object")
                 #NOTE: doing this for security purposes, why would you ever want a fixed salt?
         value = _parse_policy_value(cat, name, opt, value)
         yield cat, name, opt, value
@@ -143,7 +143,7 @@ class CryptPolicy(object):
         """
         p = ConfigParser()
         if not p.read([path]):
-            raise EnvironmentError, "failed to read config file"
+            raise EnvironmentError("failed to read config file")
         return cls(**dict(p.items(section)))
 
     @classmethod
@@ -189,7 +189,7 @@ class CryptPolicy(object):
             else: #other strings should be filepath
                 return cls.from_path(source)
         else:
-            raise TypeError, "source must be CryptPolicy, dict, config string, or file path: %r" % (type(source),)
+            raise TypeError("source must be CryptPolicy, dict, config string, or file path: %r" % (type(source),))
 
     @classmethod
     def from_sources(cls, sources):
@@ -207,7 +207,7 @@ class CryptPolicy(object):
         #check for no sources - should we return blank policy in that case?
         if len(sources) == 0:
             #XXX: er, would returning an empty policy be the right thing here?
-            raise ValueError, "no sources specified"
+            raise ValueError("no sources specified")
 
         #check if only one source
         if len(sources) == 1:
@@ -313,11 +313,11 @@ class CryptPolicy(object):
                 handler = get_crypt_handler(scheme)
             name = handler.name
             if not name:
-                raise TypeError, "handler lacks name: %r" % (handler,)
+                raise TypeError("handler lacks name: %r" % (handler,))
 
             #check name hasn't been re-used
             if name in seen:
-                raise KeyError, "multiple handlers with same name: %r" % (name,)
+                raise KeyError("multiple handlers with same name: %r" % (name,))
             seen.add(name)
 
             #add to handler list
@@ -340,7 +340,7 @@ class CryptPolicy(object):
                 if handlers:
                     for scheme in deps:
                         if scheme not in seen:
-                            raise KeyError, "known scheme in deprecated list: %r" % (scheme,)
+                            raise KeyError("known scheme in deprecated list: %r" % (scheme,))
                 dmap[cat] = frozenset(deps)
 
             #default scheme
@@ -350,7 +350,7 @@ class CryptPolicy(object):
                     if hasattr(fb, "name"):
                         fb = fb.name
                     if fb not in seen:
-                        raise KeyError, "unknown scheme set as default: %r" % (fb,)
+                        raise KeyError("unknown scheme set as default: %r" % (fb,))
                     fmap[cat] = self.get_handler(fb, required=True)
                 else:
                     fmap[cat] = fb
@@ -409,9 +409,9 @@ class CryptPolicy(object):
                 handlers = self._handlers
                 if handlers:
                     return handlers[0]
-                raise KeyError, "no crypt algorithms supported"
+                raise KeyError("no crypt algorithms supported")
         if required:
-            raise KeyError, "no crypt algorithm by that name: %r" % (name,)
+            raise KeyError("no crypt algorithm by that name: %r" % (name,))
         return None
 
     def get_options(self, name, category=None):
@@ -479,7 +479,7 @@ class CryptPolicy(object):
         mvmap = self._min_verify_time
         if category in mvmap:
             return mvmap[category]
-        elif category and None in mvap:
+        elif category and None in mvmap:
             return mvmap[None]
         else:
             return 0
@@ -583,7 +583,7 @@ class CryptPolicy(object):
     ##    p = ConfigParser()
     ##    if update and os.path.exists(path):
     ##        if not p.read([path]):
-    ##            raise EnvironmentError, "failed to read existing file"
+    ##            raise EnvironmentError("failed to read existing file")
     ##        p.remove_section(section)
     ##    self._write_to_parser(p, section)
     ##    fh = file(path, "w")
@@ -661,8 +661,6 @@ class CryptContext(object):
             policy = CryptPolicy(**kwds)
         elif kwds:
             policy = policy.replace(**kwds)
-        if not policy.has_schemes():
-            raise ValueError, "at least one scheme must be specified"
         self.policy = policy
 
     def __repr__(self):
@@ -671,7 +669,16 @@ class CryptContext(object):
         return "<CryptContext %0xd schemes=%r>" % (id(self), names)
 
     def replace(self, **kwds):
-        "returns new CryptContext with specified options modified from original; similar to CryptPolicy.replace"
+        """return mutated CryptContext instance
+
+        this function operates much like :meth:`datetime.replace()` - it returns
+        a new CryptContext instance whose configuration is exactly the
+        same as the original, with the exception of any keywords
+        specificed taking precedence over the original settings.
+
+        this is identical to the operation ``CryptContext(policy=self.policy.replace(**kwds))``,
+        see :meth:`CryptPolicy.replace` for more details.
+        """
         return CryptContext(policy=self.policy.replace(**kwds))
 
     #===================================================================
@@ -842,7 +849,7 @@ class CryptContext(object):
         #NOTE: this doesn't use category in any way, but accepts it for consistency
         if hash is None:
             if required:
-                raise ValueError, "no hash specified"
+                raise ValueError("no hash specified")
             return None
         handler = None
         for handler in self.policy.iter_handlers():
@@ -853,8 +860,8 @@ class CryptContext(object):
                     return handler.name
         if required:
             if handler is None:
-                raise KeyError, "no crypt algorithms supported"
-            raise ValueError, "hash could not be identified"
+                raise KeyError("no crypt algorithms supported")
+            raise ValueError("hash could not be identified")
         return None
 
     def encrypt(self, secret, scheme=None, category=None, **kwds):
@@ -936,20 +943,83 @@ class CryptContext(object):
 
         return result
 
-    #TODO: check this works properly, and expose it to ease requirements for apps to use migration features
-    ##def verify_and_update(self, secret, hash, scheme=None, category=None, **kwds):
-    ##    ok = self.verify(secret, hash, scheme=scheme, category=category, **kwds)
-    ##    if not ok:
-    ##        return False, None
-    ##    if self.hash_needs_update(secret, hash, category=category):
-    ##        return True, self.encrypt(secret, **kwds)
-    ##    else:
-    ##        return True, None
+    def verify_and_update(self, secret, hash, scheme=None, category=None, **kwds):
+        ok = self.verify(secret, hash, scheme=scheme, category=category, **kwds)
+        if not ok:
+            return False, None
+        if self.hash_needs_update(hash, category=category):
+            return True, self.encrypt(secret, category=category, **kwds)
+        else:
+            return True, None
 
     #=========================================================
     #eoc
     #=========================================================
 
+class LazyCryptContext(CryptContext):
+    """CryptContext subclass which doesn't load handlers until needed.
+
+    This is a subclass of CryptContext which takes in a set of arguments
+    exactly like CryptContext, but won't load any handlers
+    (or even parse it's arguments) until
+    the first time one of it's methods is accessed.
+
+    :arg schemes:
+        the first positional argument can be a list of schemes, or omitted,
+        just like CryptContext.
+
+    :param create_policy:
+
+        if a callable is passed in via this keyword,
+        it will be invoked at lazy-load time
+        with the following signature:
+        ``create_policy(**kwds) -> CryptPolicy``;
+        where ``kwds`` is all the additional kwds passed to LazyCryptContext.
+        It should return a CryptPolicy instance, which will then be used
+        by the CryptContext.
+
+    :param kwds:
+
+        All additional keywords are passed to CryptPolicy;
+        or to the create_policy function if provided.
+
+    This is mainly used internally by modules such as :mod:`passlib.apps`,
+    which define a large number of contexts, but only a few of them will be needed
+    at any one time. Use of this class saves the memory needed to import
+    the specified handlers until the context instance is actually accessed.
+    As well, it allows constructing a context at *module-init* time,
+    but using :func:`!create_policy()` to provide dynamic configuration
+    at *application-run* time.
+    """
+    _lazy_kwds = None
+
+    def __init__(self, schemes=None, **kwds):
+        if schemes is not None:
+            kwds['schemes'] = schemes
+        self._lazy_kwds = kwds
+
+    def _lazy_init(self):
+        kwds = self._lazy_kwds
+        del self._lazy_kwds
+        if 'create_policy' in kwds:
+            create_policy = kwds.pop("create_policy")
+            kwds = dict(policy=create_policy(**kwds))
+        super(LazyCryptContext, self).__init__(**kwds)
+
+    #NOTE: 'policy' property calls _lazy_init the first time it's accessed,
+    #      and relies on CryptContext.__init__ to replace it with an actual instance.
+    #      it should then have no more effect from then on.
+    class _PolicyProperty(object):
+
+        def __get__(self, obj, cls):
+            if obj is None:
+                return self
+            obj._lazy_init()
+            assert isinstance(obj.policy, CryptPolicy)
+            return obj.policy
+
+    policy = _PolicyProperty()
+
 #=========================================================
 # eof
 #=========================================================
