@@ -58,8 +58,7 @@ import logging; log = logging.getLogger(__name__)
 from warnings import warn
 #site
 #libs
-from passlib.utils import h64, classproperty, os_crypt
-from passlib.utils.handlers import MultiBackendHandler, ExtendedHandler
+from passlib.utils import h64, classproperty, os_crypt, handlers as uh
 from passlib.utils.des import mdes_encrypt_int_block
 #pkg
 #local
@@ -89,7 +88,7 @@ def raw_crypt(secret, salt):
     try:
         salt_value = h64.decode_int12(salt)
     except ValueError: #pragma: no cover - always caught by class
-        raise ValueError, "invalid chars in salt"
+        raise ValueError("invalid chars in salt")
     #FIXME: ^ this will throws error if bad salt chars are used
     # whereas linux crypt does something (inexplicable) with it
 
@@ -109,13 +108,13 @@ def raw_ext_crypt(secret, rounds, salt):
     try:
         salt_value = h64.decode_int24(salt)
     except ValueError: #pragma: no cover - always caught by class
-        raise ValueError, "invalid salt"
+        raise ValueError("invalid salt")
 
     #validate secret
     if '\x00' in secret: #pragma: no cover - always caught by class
         #builtin linux crypt doesn't like this, so we don't either
         #XXX: would make more sense to raise ValueError, but want to be compatible w/ stdlib crypt
-        raise ValueError, "secret must be string without null bytes"
+        raise ValueError("secret must be string without null bytes")
 
     #convert secret string into an integer
     key_value = _crypt_secret_to_key(secret)
@@ -136,7 +135,7 @@ def raw_ext_crypt(secret, rounds, salt):
 #=========================================================
 #handler
 #=========================================================
-class des_crypt(MultiBackendHandler):
+class des_crypt(uh.HasManyBackends, uh.HasSalt, uh.GenericHandler):
     """This class implements the des-crypt password hash, and follows the :ref:`password-hash-api`.
 
     It supports a fixed-length salt.
@@ -150,7 +149,7 @@ class des_crypt(MultiBackendHandler):
 
     It will use the first available of two possible backends:
 
-    * stdlib :func:`crypt()`, if the host OS supports des-crypt.
+    * stdlib :func:`crypt()`, if the host OS supports des-crypt (most unix systems).
     * a pure python implementation of des-crypt
 
     You can see which backend is in use by calling the :meth:`get_backend()` method.
@@ -159,9 +158,13 @@ class des_crypt(MultiBackendHandler):
     #=========================================================
     #class attrs
     #=========================================================
+    #--GenericHandler--
     name = "des_crypt"
     setting_kwds = ("salt",)
-    min_salt_chars = max_salt_chars = 2
+
+    #--HasSalt--
+    min_salt_size = max_salt_size = 2
+    salt_chars = uh.H64_CHARS
 
     #=========================================================
     #formatting
@@ -181,11 +184,10 @@ class des_crypt(MultiBackendHandler):
     @classmethod
     def from_string(cls, hash):
         if not hash:
-            raise ValueError, "no hash specified"
-        m = cls._pat.match(hash)
-        if not m:
-            raise ValueError, "invalid des-crypt hash"
-        salt, chk = m.group("salt", "chk")
+            raise ValueError("no hash specified")
+        if isinstance(hash, unicode):
+            hash = hash.encode("ascii")
+        salt, chk = hash[:2], hash[2:]
         return cls(salt=salt, checksum=chk, strict=bool(chk))
 
     def to_string(self):
@@ -205,7 +207,7 @@ class des_crypt(MultiBackendHandler):
     def _calc_checksum_builtin(self, secret):
         #forbidding nul chars because linux crypt (and most C implementations) won't accept it either.
         if '\x00' in secret:
-            raise ValueError, "null char in secret"
+            raise ValueError("null char in secret")
         #gotta do something - no official policy since des-crypt predates unicode
         if isinstance(secret, unicode):
             secret = secret.encode("utf-8")
@@ -214,7 +216,7 @@ class des_crypt(MultiBackendHandler):
     def _calc_checksum_os_crypt(self, secret):
         #os_crypt() would raise less useful error
         if '\x00' in secret:
-            raise ValueError, "null char in secret"
+            raise ValueError("null char in secret")
         #gotta do something - no official policy since des-crypt predates unicode
         if isinstance(secret, unicode):
             secret = secret.encode("utf-8")
@@ -232,7 +234,7 @@ class des_crypt(MultiBackendHandler):
 # so as not to reveal weak des keys. given the random salt, this shouldn't be
 # a very likely issue anyways, but should do something about default rounds generation anyways.
 
-class bsdi_crypt(ExtendedHandler):
+class bsdi_crypt(uh.HasManyBackends, uh.HasRounds, uh.HasSalt, uh.GenericHandler):
     """This class implements the BSDi-Crypt password hash, and follows the :ref:`password-hash-api`.
 
     It supports a fixed-length salt, and a variable number of rounds.
@@ -247,23 +249,32 @@ class bsdi_crypt(ExtendedHandler):
     :param rounds:
         Optional number of rounds to use.
         Defaults to 5000, must be between 0 and 16777215, inclusive.
+
+    It will use the first available of two possible backends:
+
+    * stdlib :func:`crypt()`, if the host OS supports bsdi-crypt (most BSD systems).
+    * a pure python implementation of bsdi-crypt
+
+    You can see which backend is in use by calling the :meth:`get_backend()` method.
     """
     #=========================================================
     #class attrs
     #=========================================================
+    #--GenericHandler--
     name = "bsdi_crypt"
     setting_kwds = ("salt", "rounds")
+    checksum_size = 11
 
-    min_salt_chars = max_salt_chars = 4
+    #--HasSalt--
+    min_salt_size = max_salt_size = 4
+    salt_chars = uh.H64_CHARS
 
+    #--HasRounds--
     default_rounds = 5000
     min_rounds = 0
     max_rounds = 16777215 # (1<<24)-1
     rounds_cost = "linear"
 
-    checksum_chars = 11
-    checksum_charset = h64.CHARS
-
     # NOTE: OpenBSD login.conf reports 7250 as minimum allowed rounds,
     # but that seems to be an OS policy, not a algorithm limitation.
 
@@ -285,10 +296,12 @@ class bsdi_crypt(ExtendedHandler):
     @classmethod
     def from_string(cls, hash):
         if not hash:
-            raise ValueError, "no hash specified"
+            raise ValueError("no hash specified")
+        if isinstance(hash, unicode):
+            hash = hash.encode("ascii")
         m = cls._pat.match(hash)
         if not m:
-            raise ValueError, "invalid ext-des-crypt hash"
+            raise ValueError("invalid ext-des-crypt hash")
         rounds, salt, chk = m.group("rounds", "salt", "chk")
         return cls(
             rounds=h64.decode_int24(rounds),
@@ -303,13 +316,25 @@ class bsdi_crypt(ExtendedHandler):
     #=========================================================
     #backend
     #=========================================================
-    #TODO: check if os_crypt supports bsdi-crypt.
+    backends = ("os_crypt", "builtin")
 
-    def calc_checksum(self, secret):
+    _has_backend_builtin = True
+
+    @classproperty
+    def _has_backend_os_crypt(cls):
+        h = '_/...lLDAxARksGCHin.'
+        return os_crypt is not None and os_crypt("test", h) == h
+
+    def _calc_checksum_builtin(self, secret):
         if isinstance(secret, unicode):
             secret = secret.encode("utf-8")
         return raw_ext_crypt(secret, self.rounds, self.salt)
 
+    def _calc_checksum_os_crypt(self, secret):
+        if isinstance(secret, unicode):
+            secret = secret.encode("utf-8")
+        return os_crypt(secret, self.to_string())[9:]
+
     #=========================================================
     #eoc
     #=========================================================
@@ -317,7 +342,7 @@ class bsdi_crypt(ExtendedHandler):
 #=========================================================
 #
 #=========================================================
-class bigcrypt(ExtendedHandler):
+class bigcrypt(uh.HasSalt, uh.GenericHandler):
     """This class implements the BigCrypt password hash, and follows the :ref:`password-hash-api`.
 
     It supports a fixed-length salt.
@@ -332,14 +357,16 @@ class bigcrypt(ExtendedHandler):
     #=========================================================
     #class attrs
     #=========================================================
+    #--GenericHandler--
     name = "bigcrypt"
     setting_kwds = ("salt",)
-
-    min_salt_chars = max_salt_chars = 2
-
-    checksum_charset = h64.CHARS
+    checksum_chars = h64.CHARS
     #NOTE: checksum chars must be multiple of 11
 
+    #--HasSalt--
+    min_salt_size = max_salt_size = 2
+    salt_chars = uh.H64_CHARS
+
     #=========================================================
     #internal helpers
     #=========================================================
@@ -356,18 +383,25 @@ class bigcrypt(ExtendedHandler):
     @classmethod
     def from_string(cls, hash):
         if not hash:
-            raise ValueError, "no hash specified"
+            raise ValueError("no hash specified")
+        if isinstance(hash, unicode):
+            hash = hash.encode("ascii")
         m = cls._pat.match(hash)
         if not m:
-            raise ValueError, "invalid bigcrypt hash"
+            raise ValueError("invalid bigcrypt hash")
         salt, chk = m.group("salt", "chk")
-        if chk and len(chk) % 11:
-            raise ValueError, "invalid bigcrypt hash"
         return cls(salt=salt, checksum=chk, strict=bool(chk))
 
     def to_string(self):
         return "%s%s" % (self.salt, self.checksum or '')
 
+    @classmethod
+    def norm_checksum(cls, value, strict=False):
+        value = super(bigcrypt, cls).norm_checksum(value, strict=strict)
+        if value and len(value) % 11:
+            raise ValueError("invalid bigcrypt hash")
+        return value
+
     #=========================================================
     #backend
     #=========================================================
@@ -392,7 +426,7 @@ class bigcrypt(ExtendedHandler):
 #=========================================================
 #
 #=========================================================
-class crypt16(ExtendedHandler):
+class crypt16(uh.HasSalt, uh.GenericHandler):
     """This class implements the crypt16 password hash, and follows the :ref:`password-hash-api`.
 
     It supports a fixed-length salt.
@@ -407,13 +441,14 @@ class crypt16(ExtendedHandler):
     #=========================================================
     #class attrs
     #=========================================================
+    #--GenericHandler--
     name = "crypt16"
     setting_kwds = ("salt",)
+    checksum_size = 22
 
-    min_salt_chars = max_salt_chars = 2
-
-    checksum_chars = 22
-    checksum_charset = h64.CHARS
+    #--HasSalt--
+    min_salt_size = max_salt_size = 2
+    salt_chars = uh.H64_CHARS
 
     #=========================================================
     #internal helpers
@@ -431,10 +466,12 @@ class crypt16(ExtendedHandler):
     @classmethod
     def from_string(cls, hash):
         if not hash:
-            raise ValueError, "no hash specified"
+            raise ValueError("no hash specified")
+        if isinstance(hash, unicode):
+            hash = hash.encode("ascii")
         m = cls._pat.match(hash)
         if not m:
-            raise ValueError, "invalid crypt16 hash"
+            raise ValueError("invalid crypt16 hash")
         salt, chk = m.group("salt", "chk")
         return cls(salt=salt, checksum=chk, strict=bool(chk))
 
@@ -454,7 +491,7 @@ class crypt16(ExtendedHandler):
         try:
             salt_value = h64.decode_int12(self.salt)
         except ValueError: #pragma: no cover - caught by class
-            raise ValueError, "invalid chars in salt"
+            raise ValueError("invalid chars in salt")
 
         #convert first 8 byts of secret string into an integer,
         key1 = _crypt_secret_to_key(secret)
@@ -468,6 +505,7 @@ class crypt16(ExtendedHandler):
         #run data through des using input of 0
         result2 = mdes_encrypt_int_block(key2, 0, salt_value, 5)
 
+        #done
         return h64.encode_dc_int64(result1) + h64.encode_dc_int64(result2)
 
     #=========================================================
