@@ -13,9 +13,11 @@ import sys
 #site
 #pkg
 from passlib import hash
-from passlib.context import CryptContext, CryptPolicy
-from passlib.utils.handlers import SimpleHandler
+from passlib.context import CryptContext, CryptPolicy, LazyCryptContext
+import passlib.utils.handlers as uh
 from passlib.tests.utils import TestCase, mktemp, catch_warnings
+from passlib.registry import register_crypt_handler_path, has_crypt_handler, \
+        _unload_handler_name as unload_handler_name
 #module
 log = getLogger(__name__)
 
@@ -161,10 +163,10 @@ admin.sha512_crypt.max_rounds = 40000
             )
 
         #check with bad handler
-        self.assertRaises(TypeError, CryptPolicy, schemes=[SimpleHandler])
+        self.assertRaises(TypeError, CryptPolicy, schemes=[uh.StaticHandler])
 
         #check with multiple handlers
-        class dummy_1(SimpleHandler):
+        class dummy_1(uh.StaticHandler):
             name = 'dummy_1'
         self.assertRaises(KeyError, CryptPolicy, schemes=[dummy_1, dummy_1])
 
@@ -370,6 +372,23 @@ admin.sha512_crypt.max_rounds = 40000
         self.assertTrue(pb.handler_is_deprecated("des_crypt", "admin"))
         self.assertFalse(pb.handler_is_deprecated("bsdi_crypt", "admin"))
 
+    def test_15_min_verify_time(self):
+        pa = CryptPolicy()
+        self.assertEqual(pa.get_min_verify_time(), 0)
+        self.assertEqual(pa.get_min_verify_time('admin'), 0)
+
+        pb = pa.replace(min_verify_time=.1)
+        self.assertEqual(pb.get_min_verify_time(), .1)
+        self.assertEqual(pb.get_min_verify_time('admin'), .1)
+
+        pc = pa.replace(admin__context__min_verify_time=.2)
+        self.assertEqual(pc.get_min_verify_time(), 0)
+        self.assertEqual(pc.get_min_verify_time('admin'), .2)
+
+        pd = pb.replace(admin__context__min_verify_time=.2)
+        self.assertEqual(pd.get_min_verify_time(), .1)
+        self.assertEqual(pd.get_min_verify_time('admin'), .2)
+
     #TODO: test this.
     ##def test_gen_min_verify_time(self):
     ##    "test get_min_verify_time() method"
@@ -447,8 +466,13 @@ class CryptContextTest(TestCase):
     def test_02_no_handlers(self):
         "test no handlers"
 
-        self.assertRaises(ValueError, CryptContext, [])
+        #check constructor...
+        cc = CryptContext()
+        self.assertRaises(KeyError, cc.identify, 'hash', required=True)
+        self.assertRaises(KeyError, cc.encrypt, 'secret')
+        self.assertRaises(KeyError, cc.verify, 'secret', 'hash')
 
+        #check updating policy after the fact...
         cc = CryptContext(['md5_crypt'])
         p = CryptPolicy(schemes=[])
         cc.policy = p
@@ -691,17 +715,90 @@ class CryptContextTest(TestCase):
         d = time.time()-s
         self.assertTrue(d>=.09,d)
         self.assertTrue(d<.5)
-        
+
         #this may take longer, so we just check min
         s = time.time()
         cc.verify("password", '_2b..iHVSUNMkJT.GcFU')
         d = time.time()-s
         self.assertTrue(d>=.09, "mvt=.1, delta=%r" % (d,))
 
+    def test_25_verify_and_update(self):
+        "test verify_and_update()"
+        cc = CryptContext(**self.sample_policy_1)
+
+        #create some hashes
+        h1 = cc.encrypt("password", scheme="des_crypt")
+        h2 = cc.encrypt("password", scheme="sha256_crypt")
+
+        #check bad password, deprecated hash
+        ok, new_hash = cc.verify_and_update("wrongpass", h1)
+        self.assertFalse(ok)
+        self.assertIs(new_hash, None)
+
+        #check bad password, good hash
+        ok, new_hash = cc.verify_and_update("wrongpass", h2)
+        self.assertFalse(ok)
+        self.assertIs(new_hash, None)
+
+        #check right password, deprecated hash
+        ok, new_hash = cc.verify_and_update("password", h1)
+        self.assertTrue(ok)
+        self.assertTrue(cc.identify(new_hash), "sha256_crypt")
+
+        #check right password, good hash
+        ok, new_hash = cc.verify_and_update("password", h2)
+        self.assertTrue(ok)
+        self.assertIs(new_hash, None)
+
     #=========================================================
     #eoc
     #=========================================================
 
+#=========================================================
+#LazyCryptContext
+#=========================================================
+class dummy_2(uh.StaticHandler):
+    name = "dummy_2"
+
+class LazyCryptContextTest(TestCase):
+    case_prefix = "LazyCryptContext"
+
+    def setUp(self):
+        unload_handler_name("dummy_2")
+
+    def tearDown(self):
+        unload_handler_name("dummy_2")
+
+    def test_kwd_constructor(self):
+        self.assertFalse(has_crypt_handler("dummy_2"))
+        register_crypt_handler_path("dummy_2", "passlib.tests.test_context")
+
+        cc = LazyCryptContext(iter(["dummy_2", "des_crypt"]), deprecated=["des_crypt"])
+
+        self.assertFalse(has_crypt_handler("dummy_2", True))
+
+        self.assertTrue(cc.policy.handler_is_deprecated("des_crypt"))
+        self.assertEqual(cc.policy.schemes(), ["dummy_2", "des_crypt"])
+
+        self.assertTrue(has_crypt_handler("dummy_2", True))
+
+    def test_callable_constructor(self):
+        self.assertFalse(has_crypt_handler("dummy_2"))
+        register_crypt_handler_path("dummy_2", "passlib.tests.test_context")
+
+        def create_policy(flag=False):
+            self.assertTrue(flag)
+            return CryptPolicy(schemes=iter(["dummy_2", "des_crypt"]), deprecated=["des_crypt"])
+
+        cc = LazyCryptContext(create_policy=create_policy, flag=True)
+
+        self.assertFalse(has_crypt_handler("dummy_2", True))
+
+        self.assertTrue(cc.policy.handler_is_deprecated("des_crypt"))
+        self.assertEqual(cc.policy.schemes(), ["dummy_2", "des_crypt"])
+
+        self.assertTrue(has_crypt_handler("dummy_2", True))
+
 #=========================================================
 #EOF
 #=========================================================
