@@ -2,10 +2,12 @@
 #=========================================================
 #imports
 #=========================================================
+from __future__ import with_statement
 #core
 from binascii import hexlify, unhexlify
 import sys
 import random
+import warnings
 #site
 #pkg
 #module
@@ -13,7 +15,7 @@ from passlib import utils
 from passlib.context import CryptContext
 from passlib.utils import h64, des, Undef, sys_bits
 from passlib.utils.md4 import md4
-from passlib.tests.utils import TestCase, Params as ak, enable_option
+from passlib.tests.utils import TestCase, Params as ak, enable_option, catch_warnings
 
 #=========================================================
 #byte funcs
@@ -31,77 +33,83 @@ class UtilsTest(TestCase):
         self.assertTrue(Undef!=True,)
 
     def test_list_to_bytes(self):
-        self.assertFunctionResults(utils.list_to_bytes, [
-            #standard big endian
-            ak('\x00', [0], 1),
-            ak('\x01', [1], 1),
-            ak('\x00\x01', [1], 2),
-            ak('\x00\x01', [0, 1], 2),
-            ak('\x00\x00\x01', [1], 3),
-            ak('\x00\x00\x00\x00', [0], 4),
-            ak('\x00\x00\x00\x01', [1], 4),
-            ak('\x00\x00\x00\xff', [255], 4),
-            ak('\x00\x00\x01\xff', [1, 255], 4),
-            ak('\x04\x03\x02\x01', [4, 3, 2, 1], 4),
+        with catch_warnings():
+            warnings.filterwarnings("ignore", r"list_to_bytes\(\) is deprecated")
 
-            #standard little endian
-            ak('\x00', [0], 1, order="little"),
-            ak('\x01', [1], 1, order="little"),
-            ak('\x01\x00', [1], 2, order="little"),
-            ak('\x01\x00', [0, 1], 2, order="little"),
-            ak('\x01\x00\x00', [1], 3, order="little"),
-            ak('\x00\x00\x00\x00', [0], 4, order="little"),
-            ak('\x01\x00\x00\x00', [1], 4, order="little"),
-            ak('\xff\x00\x00\x00', [255], 4, order="little"),
-            ak('\xff\x01\x00\x00', [1, 255], 4, order="little"),
-            ak('\x01\x02\x03\x04', [4, 3, 2, 1], 4, order="little"),
+            self.assertFunctionResults(utils.list_to_bytes, [
+                #standard big endian
+                ak('\x00', [0], 1),
+                ak('\x01', [1], 1),
+                ak('\x00\x01', [1], 2),
+                ak('\x00\x01', [0, 1], 2),
+                ak('\x00\x00\x01', [1], 3),
+                ak('\x00\x00\x00\x00', [0], 4),
+                ak('\x00\x00\x00\x01', [1], 4),
+                ak('\x00\x00\x00\xff', [255], 4),
+                ak('\x00\x00\x01\xff', [1, 255], 4),
+                ak('\x04\x03\x02\x01', [4, 3, 2, 1], 4),
 
-            ])
+                #standard little endian
+                ak('\x00', [0], 1, order="little"),
+                ak('\x01', [1], 1, order="little"),
+                ak('\x01\x00', [1], 2, order="little"),
+                ak('\x01\x00', [0, 1], 2, order="little"),
+                ak('\x01\x00\x00', [1], 3, order="little"),
+                ak('\x00\x00\x00\x00', [0], 4, order="little"),
+                ak('\x01\x00\x00\x00', [1], 4, order="little"),
+                ak('\xff\x00\x00\x00', [255], 4, order="little"),
+                ak('\xff\x01\x00\x00', [1, 255], 4, order="little"),
+                ak('\x01\x02\x03\x04', [4, 3, 2, 1], 4, order="little"),
 
-        #check bytes size check
-        self.assertRaises(ValueError, utils.list_to_bytes, [])
-        self.assertRaises(ValueError, utils.list_to_bytes, [], bytes=0)
-        self.assertRaises(ValueError, utils.list_to_bytes, [0, 0], bytes=1)
+                ])
 
-        #check bytes bound check
-        self.assertRaises(ValueError, utils.list_to_bytes, [256], bytes=1)
+            #check bytes size check
+            self.assertRaises(ValueError, utils.list_to_bytes, [])
+            self.assertRaises(ValueError, utils.list_to_bytes, [], count=0)
+            self.assertRaises(ValueError, utils.list_to_bytes, [0, 0], count=1)
 
-        #quick check native mode works right
-        if sys.byteorder == "little":
-            self.assertEqual(utils.list_to_bytes([1], 3, order="native"), '\x01\x00\x00')
-        else:
-            self.assertEqual(utils.list_to_bytes([1], 3, order="native"), '\x00\x00\x01')
+            #check bytes bound check
+            self.assertRaises(ValueError, utils.list_to_bytes, [256], count=1)
+
+            #quick check native mode works right
+            if sys.byteorder == "little":
+                self.assertEqual(utils.list_to_bytes([1], 3, order="native"), '\x01\x00\x00')
+            else:
+                self.assertEqual(utils.list_to_bytes([1], 3, order="native"), '\x00\x00\x01')
 
     def test_bytes_to_list(self):
-        self.assertFunctionResults(utils.bytes_to_list, [
+        with catch_warnings():
+            warnings.filterwarnings("ignore", r"bytes_to_list\(\) is deprecated")
 
-            #standard big endian
-            ak([1], '\x01'),
-            ak([0, 1], '\x00\x01'),
-            ak([0, 0, 1], '\x00\x00\x01'),
-            ak([0, 0, 0, 0],'\x00\x00\x00\x00'),
-            ak([0, 0, 0, 1],'\x00\x00\x00\x01'),
-            ak([0, 0, 0, 255],'\x00\x00\x00\xff'),
-            ak([0, 0, 1, 0],'\x00\x00\x01\x00'),
-            ak([4, 3, 2, 1],'\x04\x03\x02\x01'),
+            self.assertFunctionResults(utils.bytes_to_list, [
 
-            #standard little endian
-            ak([1], '\x01', order="little"),
-            ak([0, 1], '\x01\x00', order="little"),
-            ak([0, 0, 1], '\x01\x00\x00', order="little"),
-            ak([0, 0, 0, 0], '\x00\x00\x00\x00', order="little"),
-            ak([0, 0, 0, 1], '\x01\x00\x00\x00', order="little"),
-            ak([0, 0, 0, 255], '\xff\x00\x00\x00', order="little"),
-            ak([0, 0, 1, 0], '\x00\x01\x00\x00', order="little"),
-            ak([4, 3, 2, 1],'\x01\x02\x03\x04', order="little"),
+                #standard big endian
+                ak([1], '\x01'),
+                ak([0, 1], '\x00\x01'),
+                ak([0, 0, 1], '\x00\x00\x01'),
+                ak([0, 0, 0, 0],'\x00\x00\x00\x00'),
+                ak([0, 0, 0, 1],'\x00\x00\x00\x01'),
+                ak([0, 0, 0, 255],'\x00\x00\x00\xff'),
+                ak([0, 0, 1, 0],'\x00\x00\x01\x00'),
+                ak([4, 3, 2, 1],'\x04\x03\x02\x01'),
 
-            ])
+                #standard little endian
+                ak([1], '\x01', order="little"),
+                ak([0, 1], '\x01\x00', order="little"),
+                ak([0, 0, 1], '\x01\x00\x00', order="little"),
+                ak([0, 0, 0, 0], '\x00\x00\x00\x00', order="little"),
+                ak([0, 0, 0, 1], '\x01\x00\x00\x00', order="little"),
+                ak([0, 0, 0, 255], '\xff\x00\x00\x00', order="little"),
+                ak([0, 0, 1, 0], '\x00\x01\x00\x00', order="little"),
+                ak([4, 3, 2, 1],'\x01\x02\x03\x04', order="little"),
 
-        #quick check native mode works right
-        if sys.byteorder == "little":
-            self.assertEqual(utils.bytes_to_list('\x01\x00\x00', order="native"), [0, 0, 1])
-        else:
-            self.assertEqual(utils.bytes_to_list('\x00\x00\x01', order="native"), [0, 0, 1])
+                ])
+
+            #quick check native mode works right
+            if sys.byteorder == "little":
+                self.assertEqual(utils.bytes_to_list('\x01\x00\x00', order="native"), [0, 0, 1])
+            else:
+                self.assertEqual(utils.bytes_to_list('\x00\x00\x01', order="native"), [0, 0, 1])
 
     def test_getrandbytes(self):
         def f(*a,**k):
@@ -135,6 +143,9 @@ class UtilsTest(TestCase):
         self.assertNotEqual(a,b)
         self.assertEqual(sorted(set(a)), ['a','b','c'])
 
+        #generate_password
+        self.assertEqual(len(utils.generate_password(15)), 15)
+
     def test_is_crypt_context(self):
         cc = CryptContext(["des_crypt"])
         self.assertTrue(utils.is_crypt_context(cc))
@@ -468,6 +479,45 @@ class _Pbkdf2BackendTest(TestCase):
             ),
         ])
 
+    def test_rfc6070(self):
+        "rfc6070 test vectors"
+        self.assertFunctionResults(pbkdf2.pbkdf2, [
+
+            (
+                unhexlify("0c60c80f961f0e71f3a9b524af6012062fe037a6"),
+                "password", "salt" , 1, 20,
+            ),
+
+            (
+                unhexlify("ea6c014dc72d6f8ccd1ed92ace1d41f0d8de8957"),
+                "password", "salt", 2, 20,
+            ),
+
+            (
+                unhexlify("4b007901b765489abead49d926f721d065a429c1"),
+                "password", "salt", 4096, 20,
+            ),
+
+            #just runs too long - could enable if ALL option is set
+            ##(
+            ##
+            ##    unhexlify("eefe3d61cd4da4e4e9945b3d6ba2158c2634e984"),
+            ##    "password", "salt", 16777216, 20,
+            ##),
+
+            (
+                unhexlify("3d2eec4fe41c849b80c8d83662c0e44a8b291a964cf2f07038"),
+                "passwordPASSWORDpassword",
+                "saltSALTsaltSALTsaltSALTsaltSALTsalt",
+                4096, 25,
+            ),
+
+            (
+                unhexlify("56fa6aa75548099dcc37d7f03425e0c3"),
+                "pass\00word", "sa\00lt", 4096, 16,
+            ),
+        ])
+
     def test_invalid_values(self):
 
         #invalid rounds
