@@ -10,6 +10,7 @@ import os
 import tempfile
 import unittest
 import warnings
+from warnings import warn
 try:
     from warnings import catch_warnings
 except ImportError:
@@ -24,8 +25,10 @@ except ImportError:
 #site
 from nose.plugins.skip import SkipTest
 #pkg
-from passlib.utils import classproperty
-from passlib.utils.handlers import SimpleHandler, ExtendedHandler, MultiBackendHandler
+from passlib import registry
+from passlib.utils import classproperty, handlers as uh, \
+        has_rounds_info, has_salt_info, \
+        rounds_cost_values
 #local
 __all__ = [
     #util funcs
@@ -139,7 +142,7 @@ class TestCase(unittest.TestCase):
 
     def assertNotEquals(self, real, correct, msg=None):
         #NOTE: overriding this to get msg formatting capability
-        msg = self._format_msg(msg, "got %r, expected would equal %r", real, correct)
+        msg = self._format_msg(msg, "got %r, expected would not equal %r", real, correct)
         return self.assert_(real != correct, msg)
 
     def assertNotEqual(self, *a, **k):
@@ -203,11 +206,20 @@ class TestCase(unittest.TestCase):
 #other unittest helpers
 #=========================================================
 class HandlerCase(TestCase):
-    """base class for testing password hash handlers (esp passlib.utils.handlers.SimpleHandler subclasses)
+    """base class for testing password hash handlers (esp passlib.utils.handlers subclasses)
+
+    In order to use this to test a handler,
+    create a subclass will all the appropriate attributes
+    filled as listed in the example below,
+    and run the subclass via unittest.
 
     .. todo::
-        write directions on how to use this class.
-        for now, see examples in test_handlers
+
+        Document all of the options HandlerCase offers.
+
+    .. note::
+
+        This is subclass of :class:`unittest.TestCase`.
     """
     #=========================================================
     #attrs to be filled in by subclass for testing specific handler
@@ -296,14 +308,6 @@ class HandlerCase(TestCase):
             name += " (%s backend)" % (backend(),)
         return name
 
-    @classproperty
-    def has_salt_info(cls):
-        return 'salt' in cls.handler.setting_kwds and getattr(cls.handler, "max_salt_chars", None) > 0
-
-    @classproperty
-    def has_rounds_info(cls):
-        return 'rounds' in cls.handler.setting_kwds and getattr(cls.handler, "max_rounds", None) > 0
-
     backend = "default"
 
     def setUp(self):
@@ -339,61 +343,91 @@ class HandlerCase(TestCase):
         self.assert_(context is not None, "context_kwds must be defined:")
         self.assertIsInstance(context, tuple, "context_kwds must be a tuple:")
 
-    #TODO: check optional rounds attributes & salt attributes
-
-    def test_04_base_handler(self):
-        "check configuration of SimpleHandler-derived classes"
+    def test_01_optional_salt_attributes(self):
+        "validate optional salt attributes"
         cls = self.handler
-        if not isinstance(cls, type) or not issubclass(cls, SimpleHandler):
+        if not has_salt_info(cls):
             raise SkipTest
 
-        if not cls.name:
-            raise AssertionError, "class must have .name attribute set"
+        #check max_salt_size
+        mx_set = (cls.max_salt_size is not None)
+        if mx_set and cls.max_salt_size < 1:
+            raise AssertionError("max_salt_chars must be >= 1")
 
-        if cls.setting_kwds is None:
-            raise AssertionError, "class must have .setting_kwds attribute set"
+        #check min_salt_size
+        if cls.min_salt_size < 0:
+            raise AssertionError("min_salt_chars must be >= 0")
+        if mx_set and cls.min_salt_size > cls.max_salt_size:
+            raise AssertionError("min_salt_chars must be <= max_salt_chars")
+
+        #check default_salt_size
+        if cls.default_salt_size < cls.min_salt_size:
+            raise AssertionError("default_salt_size must be >= min_salt_size")
+        if mx_set and cls.default_salt_size > cls.max_salt_size:
+            raise AssertionError("default_salt_size must be <= max_salt_size")
+
+        #check for 'salt_size' keyword
+        if 'salt_size' not in cls.setting_kwds and \
+                (not mx_set or cls.min_salt_size < cls.max_salt_size):
+            #NOTE: for now, only bothering to issue warning if default_salt_size isn't maxed out
+            if (not mx_set or cls.default_salt_size < cls.max_salt_size):
+                warn("%s: hash handler supports range of salt sizes, but doesn't specify 'salt_size' setting" % (cls.name,))
+
+        #check salt_chars & default_salt_chars
+        if cls.salt_chars:
+            if not cls.default_salt_chars:
+                raise AssertionError("default_salt_chars must not be empty")
+            if any(c not in cls.salt_chars for c in cls.default_salt_chars):
+                raise AssertionError("default_salt_chars must be subset of salt_chars: %r not in salt_chars" % (c,))
+        else:
+            if not cls.default_salt_chars:
+                raise AssertionError("default_salt_chars MUST be specified if salt_chars is empty")
+
+    def test_02_optional_rounds_attributes(self):
+        "validate optional rounds attributes"
+        cls = self.handler
+        if not has_rounds_info(cls):
+            raise SkipTest
+
+        #check max_rounds
+        if cls.max_rounds is None:
+            raise AssertionError("max_rounds not specified")
+        if cls.max_rounds < 1:
+            raise AssertionError("max_rounds must be >= 1")
+
+        #check min_rounds
+        if cls.min_rounds < 0:
+            raise AssertionError("min_rounds must be >= 0")
+        if cls.min_rounds > cls.max_rounds:
+            raise AssertionError("min_rounds must be <= max_rounds")
+
+        #check default_rounds
+        if cls.default_rounds is not None:
+            if cls.default_rounds < cls.min_rounds:
+                raise AssertionError("default_rounds must be >= min_rounds")
+            if cls.default_rounds > cls.max_rounds:
+                raise AssertionError("default_rounds must be <= max_rounds")
+
+        #check rounds_cost
+        if cls.rounds_cost not in rounds_cost_values:
+            raise AssertionError("unknown rounds cost constant: %r" % (cls.rounds_cost,))
 
     def test_05_ext_handler(self):
-        "check configuration of ExtendedHandler-derived classes"
+        "check configuration of GenericHandler-derived classes"
         cls = self.handler
-        if not isinstance(cls, type) or not issubclass(cls, ExtendedHandler):
+        if not isinstance(cls, type) or not issubclass(cls, uh.GenericHandler):
             raise SkipTest
 
-        if any(k not in cls.setting_kwds for k in cls._extra_init_settings):
-            raise AssertionError, "_extra_init_settings must be subset of setting_kwds"
-
-        if 'salt' in cls.setting_kwds:
-
-            if cls.min_salt_chars > cls.max_salt_chars:
-                raise AssertionError, "min salt chars too large"
-
-            if cls.default_salt_chars < cls.min_salt_chars:
-                raise AssertionError, "default salt chars too small"
-            if cls.default_salt_chars > cls.max_salt_chars:
-                raise AssertionError, "default salt chars too large"
-
-            if any(c not in cls.salt_charset for c in cls.default_salt_charset):
-                raise AssertionError, "default salt charset not subset of salt charset"
-
-        if 'rounds' in cls.setting_kwds:
-
-            if cls.max_rounds is None:
-                raise AssertionError, "max rounds not specified"
-
-            if cls.min_rounds > cls.max_rounds:
-                raise AssertionError, "min rounds too large"
-
-            if cls.default_rounds is not None:
-                if cls.default_rounds < cls.min_rounds:
-                    raise AssertionError, "default rounds too small"
-                if cls.default_rounds > cls.max_rounds:
-                    raise AssertionError, "default rounds too large"
-
-            if cls.rounds_cost not in ("linear", "log2"):
-                raise AssertionError, "unknown rounds cost function"
+        if 'ident' in cls.setting_kwds:
+            # assume uses HasManyIdents
+            self.assertTrue(len(cls.ident_values)>1, "cls.ident_values must have 2+ elements")
+            self.assertTrue(cls.default_ident in cls.ident_values, "cls.default_ident must specify member of cls.ident_values")
+            if cls.ident_aliases:
+                for alias, ident in cls.ident_aliases.iteritems():
+                    self.assertTrue(ident in cls.ident_values, "cls.ident_aliases must map to cls.ident_values members: %r" % (ident,))
 
     def test_06_backend_handler(self):
-        "check configuration of MultiBackendHandler-derived classes"
+        "check behavior of multiple-backend handlers"
         h = self.handler
         if not hasattr(h, "get_backend"):
             raise SkipTest
@@ -413,7 +447,7 @@ class HandlerCase(TestCase):
                     self.assertRaises(ValueError, h.set_backend, backend)
                 else:
                     #failure eg: used classmethod instead of classproperty in _has_backend_xxx
-                    raise TypeError, "has_backend(%r) returned invalid value: %r" % (backend, r,)
+                    raise TypeError("has_backend(%r) returned invalid value: %r" % (backend, r,))
         finally:
             h.set_backend(orig)
 
@@ -482,16 +516,21 @@ class HandlerCase(TestCase):
                 continue
             self.assertRaises(ValueError, self.do_verify, 'stub', hash, __msg__="scheme=%r, hash=%r:" % (name, hash))
 
-    def test_22_verify_invalid(self):
-        "test verify() throws error against known-invalid hashes"
-        if not self.known_unidentified_hashes and not self.known_malformed_hashes:
+    def test_22_verify_unidentified(self):
+        "test verify() throws error against known-unidentified hashes"
+        if not self.known_unidentified_hashes:
             raise SkipTest
         for hash in self.known_unidentified_hashes:
             self.assertRaises(ValueError, self.do_verify, 'stub', hash, __msg__="hash=%r:" % (hash,))
+
+    def test_23_verify_malformed(self):
+        "test verify() throws error against known-malformed hashes"
+        if not self.known_malformed_hashes:
+            raise SkipTest
         for hash in self.known_malformed_hashes:
             self.assertRaises(ValueError, self.do_verify, 'stub', hash, __msg__="hash=%r:" % (hash,))
 
-    def test_23_verify_none(self):
+    def test_24_verify_none(self):
         "test verify() throws error against hash=None/empty string"
         #find valid hash so that doesn't mask error
         self.assertRaises(ValueError, self.do_verify, 'stub', None, __msg__="hash=None:")
@@ -513,46 +552,63 @@ class HandlerCase(TestCase):
 
     def test_31_genconfig_minsalt(self):
         "test genconfig() honors min salt chars"
-        if not self.has_salt_info:
-            raise SkipTest
         handler = self.handler
-        cs = handler.salt_charset
-        mn = handler.min_salt_chars
+        if not has_salt_info(handler):
+            raise SkipTest
+        cs = handler.salt_chars
+        mn = handler.min_salt_size
         c1 = self.do_genconfig(salt=cs[0] * mn)
         if mn > 0:
             self.assertRaises(ValueError, self.do_genconfig, salt=cs[0]*(mn-1))
 
     def test_32_genconfig_maxsalt(self):
         "test genconfig() honors max salt chars"
-        if not self.has_salt_info:
-            raise SkipTest
         handler = self.handler
-        cs = handler.salt_charset
-        mx = handler.max_salt_chars
-        c1 = self.do_genconfig(salt=cs[0] * mx)
-        c2 = self.do_genconfig(salt=cs[0] * (mx+1))
-        self.assertEquals(c1,c2)
+        if not has_salt_info(handler):
+            raise SkipTest
+        cs = handler.salt_chars
+        mx = handler.max_salt_size
+        if mx is None:
+            #make sure salt is NOT truncated,
+            #use a really large salt for testing
+            salt = cs[0] * 1024
+            c1 = self.do_genconfig(salt=salt)
+            c2 = self.do_genconfig(salt=salt + cs[0])
+            self.assertNotEqual(c1,c2)
+        else:
+            #make sure salt is truncated exactly where it should be.
+            salt = cs[0] * mx
+            c1 = self.do_genconfig(salt=salt)
+            c2 = self.do_genconfig(salt=salt + cs[0])
+            self.assertEqual(c1,c2)
+
+            #if min_salt supports it, check smaller than mx is NOT truncated
+            if handler.min_salt_size < mx:
+                c3 = self.do_genconfig(salt=salt[:-1])
+                self.assertNotEqual(c1,c3)
 
     def test_33_genconfig_saltcharset(self):
         "test genconfig() honors salt charset"
-        if not self.has_salt_info:
-            raise SkipTest
         handler = self.handler
-        mx = handler.max_salt_chars
-        mn = handler.min_salt_chars
-        cs = handler.salt_charset
+        if not has_salt_info(handler):
+            raise SkipTest
+        mx = handler.max_salt_size
+        mn = handler.min_salt_size
+        cs = handler.salt_chars
 
         #make sure all listed chars are accepted
-        for i in xrange(0,len(cs),mx):
-            salt = cs[i:i+mx]
+        chunk = 1024 if mx is None else mx
+        for i in xrange(0,len(cs),chunk):
+            salt = cs[i:i+chunk]
             if len(salt) < mn:
-                salt = (salt*(mn//len(salt)+1))[:mx]
+                salt = (salt*(mn//len(salt)+1))[:chunk]
             self.do_genconfig(salt=salt)
 
-        #check some invalid salt chars are rejected
+        #check some invalid salt chars, make sure they're rejected
+        chunk = mn if mn > 0 else 1
         for c in '\x00\xff':
             if c not in cs:
-                self.assertRaises(ValueError, self.do_genconfig, salt=c*mx)
+                self.assertRaises(ValueError, self.do_genconfig, salt=c*chunk)
 
     #=========================================================
     #genhash()
@@ -674,7 +730,7 @@ class HandlerCase(TestCase):
 #=========================================================
 def enable_backend_case(handler, name):
     "helper to check if a separate test is needed for the specified backend"
-    assert issubclass(handler, MultiBackendHandler), "handler must derived from MultiBackendHandler"
+    assert hasattr(handler, "backends"), "handler must support uh.HasManyBackends protocol"
     assert name in handler.backends, "unknown backend: %r" % (name,)
     return enable_option("all-backends") and handler.get_backend() != name and handler.has_backend(name)
 
@@ -694,6 +750,27 @@ def create_backend_case(base_test, name):
     dummy.__name__ = name.title() + base_test.__name__
     return dummy
 
+#=========================================================
+#misc helpers
+#=========================================================
+class dummy_handler_in_registry(object):
+    "context manager that inserts dummy handler in registry"
+    def __init__(self, name):
+        self.name = name
+        self.dummy = type('dummy_' + name, (uh.GenericHandler,), dict(
+            name=name,
+            setting_kwds=(),
+        ))
+
+    def __enter__(self):
+        registry._unload_handler_name(self.name, locations=False)
+        registry.register_crypt_handler(self.dummy)
+        assert registry.get_crypt_handler(self.name) is self.dummy
+        return self.dummy
+
+    def __exit__(self, *exc_info):
+        registry._unload_handler_name(self.name, locations=False)
+
 #=========================================================
 #helper for creating temp files - all cleaned up when prog exits
 #=========================================================
